using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Syncfusion.Windows.Forms.Grid;

namespace SeperateModelTest
{
    public partial class GridControlFormulasTest : Form
    {
        private ExternalData _extData;

        public GridControlFormulasTest()
        {
            InitializeComponent();
        }

        private void LoadGridControlFormulasTest(object sender, EventArgs e)
        {
            gridControl1.Model.CellModels.Add("FormulaCell", new GridFormulaCellModel(gridControl1.Model));
            gridControl1.Model.BaseStylesMap["Standard"].StyleInfo.CellType = "FormulaCell";

            GridFormulaCellRenderer.ForceEditWhenActivated = false;
            gridControl1.Model.Options.ActivateCurrentCellBehavior = GridCellActivateAction.DblClickOnCell;
            gridControl1.EnterKeyBehavior = GridDirectionType.Down;

            GridFormulaEngine engine = ((GridFormulaCellModel)gridControl1.Model.CellModels["FormulaCell"]).Engine;
            engine.ForceSaveCellInfo = true;

            _extData = new ExternalData(100, 20);
            gridControl1.Model.QueryCellInfo += new Syncfusion.Windows.Forms.Grid.GridQueryCellInfoEventHandler(GridQueryCellInfo);
            gridControl1.Model.QueryRowCount += new Syncfusion.Windows.Forms.Grid.GridRowColCountEventHandler(GridQueryRowCount);
            gridControl1.Model.QueryColCount += new Syncfusion.Windows.Forms.Grid.GridRowColCountEventHandler(GridQueryColCount);
            gridControl1.Model.SaveCellInfo += new GridSaveCellInfoEventHandler(GridSaveCellInfo);

            gridControl1.HorizontalScroll += new ScrollEventHandler(horizontalScroll);

            gridControl1.Model.ResetVolatileData();
        }

        private void GridQueryCellInfo(object sender, GridQueryCellInfoEventArgs e)
        {
            if (e.RowIndex > 0 && e.ColIndex > 0)
            {
                e.Style.CellValue = _extData[e.RowIndex - 1, e.ColIndex - 1].value;
                e.Style.FormulaTag = _extData[e.RowIndex - 1, e.ColIndex - 1].tag;
                e.Handled = true;
            }
        }

        private void GridQueryRowCount(object sender, GridRowColCountEventArgs e)
        {
            e.Count = _extData.RowCount;
            e.Handled = true;
        }

        private void GridQueryColCount(object sender, GridRowColCountEventArgs e)
        {
            e.Count = _extData.ColCount + _extraColumns;
            e.Handled = true;
        }

        private void GridSaveCellInfo(object sender, GridSaveCellInfoEventArgs e)
        {
            try
            {
                if (e.ColIndex > 0 && e.RowIndex > 0)
                {
                    WithFormula withFormula = new WithFormula(e.Style.CellValue.ToString(), e.Style.FormulaTag);
                    _extData[e.RowIndex - 1, e.ColIndex - 1] = withFormula;
                }
            }
            catch { }
            e.Handled = true;
        }

        private int _extraColumns = 0;

        private void horizontalScroll(object sender, ScrollEventArgs e)
        {
            // These ones should not alter the number of columns in the grid. This is what Excel does
            if (e.Type == ScrollEventType.ThumbPosition || e.Type == ScrollEventType.ThumbTrack)
                return;

            // Only the columns from newValue are being displayed on the screen at this time
            int startColumn = e.NewValue;
            int columnsUsed = e.NewValue;

            // The amount of screen space taken up by the columns already in existance after startColumn
            int dataWidth = 0;

            // The amount of screen available to the grid
            int targetWidth = gridControl1.Width;
            
            // If the user scrolls left we will decrease the number of extras and this gets set to false
            bool increaseExtras = true;

            for (int i = startColumn; i < _extData.ColCount + _extraColumns; i++)
            {
                columnsUsed++;
                dataWidth += gridControl1.Cols.Size[i];

                // There are enough columns allocated to fill the screen, remove any that are surplus to requirements
                if (dataWidth >= targetWidth)
                {
                    // Remove any additional columns added last time but not used this time
                    if (columnsUsed > _extData.ColCount)
                        // Here we still need some columns beyond the real, data filled columns
                        _extraColumns = i - _extData.ColCount;
                    else
                        // Here there are enough columns to fill up the screen without adding dummy columns
                        _extraColumns = 0;
                    increaseExtras = false;
                    break;
                }
            }

            // If we get to here we need some empty columns to fill the screen. We will add them with the default width
            if (increaseExtras)
                _extraColumns += (targetWidth - dataWidth) / gridControl1.Cols.DefaultSize;

            // Refresh the grid to show the new columns
            gridControl1.BeginUpdate();
            gridControl1.Model.ResetVolatileData();
            gridControl1.Refresh();
            gridControl1.EndUpdate();
        }
    }
}